@@ -2,19 +2,22 @@ use crate::{
     context::{create_indent_trivia, create_newline_trivia, Context},
     fmt_symbol,
     formatters::{
-        expression::{format_expression, hang_expression},
+        expression::{format_expression, hang_expression, is_brackets_string},
         general::{format_contained_span, format_end_token, format_token_reference, EndTokenType},
         trivia::{strip_trivia, FormatTriviaType, UpdateLeadingTrivia, UpdateTrailingTrivia},
         trivia_util,
     },
     shape::Shape,
 };
-use full_moon::ast::{
-    punctuated::{Pair, Punctuated},
-    span::ContainedSpan,
-    Expression, Field, TableConstructor, Value,
+use full_moon::{
+    ast::{
+        punctuated::{Pair, Punctuated},
+        span::ContainedSpan,
+        Expression, Field, TableConstructor, Value,
+    },
+    node::Node,
+    tokenizer::{Token, TokenReference, TokenType},
 };
-use full_moon::tokenizer::{Token, TokenReference};
 
 /// Used to provide information about the table
 #[derive(Debug, Clone, Copy)]
@@ -27,6 +30,80 @@ pub enum TableType {
     Empty,
 }
 
+/// Formats an Expression value part of a k,v field pair
+fn format_field_expression_value(
+    ctx: &Context,
+    expression: &Expression,
+    shape: Shape,
+) -> Expression {
+    let singleline_value = format_expression(ctx, expression, shape)
+        .update_trailing_trivia(FormatTriviaType::Replace(vec![])); // We will remove all the trivia from this value, and place it after the comma
+
+    if trivia_util::can_hang_expression(expression)
+        && shape.take_first_line(&singleline_value).over_budget()
+    {
+        hang_expression(ctx, expression, shape, Some(1))
+            .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+    } else {
+        singleline_value
+    }
+}
+
+/// Handles the formatting of the comments around a key and the equals sign in a field of a table.
+/// Takes in the key as a node (so that we can handle both expression key brackets and name keys)
+/// as well as the equals sign, then outputs the new leading trivia of the key + new equals token. The trailing trivia of the key should be emptied.
+fn handle_field_key_equals_comments<T: Node>(
+    ctx: &Context,
+    key: &T,
+    equal: &TokenReference,
+    shape: Shape,
+) -> (Vec<Token>, TokenReference) {
+    // Get the current leading and trailing trivia around the key
+    let (key_leading_trivia, key_trailing_trivia) = key.surrounding_trivia();
+
+    // Take leading and trailing comments from the equal sign, and put it before the key
+    let equal_sign_comments = equal
+        .leading_trivia()
+        .chain(equal.trailing_trivia())
+        .filter(|token| trivia_util::trivia_is_comment(token));
+
+    // Join the key trailing comments with the equal sign comments, as we will move them to before the key.
+    // Also adds in the necessary whitespace trivia
+    let key_leading_comments = key_trailing_trivia
+        .iter()
+        .filter(|token| trivia_util::trivia_is_comment(token))
+        .map(|x| x.to_owned())
+        .chain(equal_sign_comments)
+        .map(|x| x.to_owned())
+        .flat_map(|trivia| {
+            // Prepend an indent before the comment, and append a newline after the comments
+            vec![
+                create_indent_trivia(ctx, shape),
+                trivia,
+                create_newline_trivia(ctx),
+            ]
+        });
+
+    // Join the new leading comments with the existing leading comments, and collect into into a Vec
+    let key_leading_comments = key_leading_trivia
+        .iter()
+        .map(|x| x.to_owned().to_owned())
+        .chain(key_leading_comments)
+        .collect::<Vec<_>>();
+
+    // Create the new equals token
+    let equal = TokenReference::symbol("=")
+        .unwrap()
+        .update_leading_trivia(FormatTriviaType::Replace(vec![Token::new(
+            TokenType::spaces(1),
+        )]))
+        .update_trailing_trivia(FormatTriviaType::Replace(vec![Token::new(
+            TokenType::spaces(1),
+        )]));
+
+    (key_leading_comments, equal)
+}
+
 fn format_field(
     ctx: &Context,
     field: &Field,
@@ -47,24 +124,35 @@ fn format_field(
             value,
         } => {
             trailing_trivia = trivia_util::get_expression_trailing_trivia(value);
-            let brackets =
-                format_contained_span(ctx, brackets, shape).update_leading_trivia(leading_trivia);
-            let key = format_expression(ctx, key, shape + 1); // 1 = opening bracket
-            let equal = fmt_symbol!(ctx, equal, " = ", shape);
-            let shape = shape.take_last_line(&key) + (2 + 3); // 2 = brackets, 3 = " = "
+            let brackets = format_contained_span(ctx, brackets, shape);
 
-            let singleline_value = format_expression(ctx, value, shape)
-                .update_trailing_trivia(FormatTriviaType::Replace(vec![])); // We will remove all the trivia from this value, and place it after the comma
-
-            let value = if trivia_util::can_hang_expression(value)
-                && shape.take_first_line(&singleline_value).over_budget()
-            {
-                hang_expression(ctx, value, shape, Some(1))
-                    .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+            let space_brackets = is_brackets_string(key);
+            let key = if space_brackets {
+                format_expression(ctx, key, shape + 2) // 2 = "[ "
+                    .update_leading_trivia(FormatTriviaType::Append(vec![Token::new(
+                        TokenType::spaces(1),
+                    )]))
+                    .update_trailing_trivia(FormatTriviaType::Append(vec![Token::new(
+                        TokenType::spaces(1),
+                    )]))
             } else {
-                singleline_value
+                format_expression(ctx, key, shape + 1) // 1 = "["
             };
 
+            // Get the new leading comments to add before the key, and the equal token
+            let (key_leading_comments, equal) =
+                handle_field_key_equals_comments(ctx, &brackets, equal, shape);
+
+            // Update the key to contain the leading comments, remove the trailing comments,
+            // and also add the extra leading_trivia we add in general to all fields
+            let brackets = brackets
+                .update_leading_trivia(FormatTriviaType::Replace(key_leading_comments))
+                .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+                .update_leading_trivia(leading_trivia);
+
+            let shape = shape.take_last_line(&key) + (2 + 3 + if space_brackets { 2 } else { 0 }); // 2 = brackets, 3 = " = ", 2 = spaces around brackets if necessary
+            let value = format_field_expression_value(ctx, value, shape);
+
             Field::ExpressionKey {
                 brackets,
                 key,
@@ -74,21 +162,21 @@ fn format_field(
         }
         Field::NameKey { key, equal, value } => {
             trailing_trivia = trivia_util::get_expression_trailing_trivia(value);
-            let key = format_token_reference(ctx, key, shape).update_leading_trivia(leading_trivia);
-            let equal = fmt_symbol!(ctx, equal, " = ", shape);
+            let key = format_token_reference(ctx, key, shape);
+
+            // Get the new leading comments to add before the key, and the equal token
+            let (key_leading_comments, equal) =
+                handle_field_key_equals_comments(ctx, &key, equal, shape);
+
+            // Update the key to contain the leading comments, remove the trailing comments,
+            // and also add the extra leading_trivia we add in general to all fields
+            let key = key
+                .update_leading_trivia(FormatTriviaType::Replace(key_leading_comments))
+                .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
+                .update_leading_trivia(leading_trivia);
+
             let shape = shape + (strip_trivia(&key).to_string().len() + 3); // 3 = " = "
-
-            let singleline_value = format_expression(ctx, value, shape)
-                .update_trailing_trivia(FormatTriviaType::Replace(vec![])); // We will remove all the trivia from this value, and place it after the comma
-
-            let value = if trivia_util::can_hang_expression(value)
-                && shape.take_first_line(&singleline_value).over_budget()
-            {
-                hang_expression(ctx, value, shape, Some(1))
-                    .update_trailing_trivia(FormatTriviaType::Replace(vec![]))
-            } else {
-                singleline_value
-            };
+            let value = format_field_expression_value(ctx, value, shape);
 
             Field::NameKey { key, equal, value }
         }
@@ -357,10 +445,16 @@ pub fn format_table_constructor(
             // exponential time complexity with respect to how deep the table is.
             // TODO: find an improved heuristic whilst comparing against benchmarks
             let braces_range = (
-                start_brace.token().end_position().bytes(),
+                // Use the position of the last trivia in case there is some present (e.g. whitespace)
+                // So that we don't include an extra space
+                if let Some(token) = start_brace.leading_trivia().last() {
+                    token.end_position().bytes()
+                } else {
+                    start_brace.token().end_position().bytes()
+                },
                 end_brace.token().start_position().bytes(),
             );
-            let singleline_shape = shape + (braces_range.1 - braces_range.0) + 4; // 4 = two braces + single space before/after them
+            let singleline_shape = shape + (braces_range.1 - braces_range.0) + 3; // 4 = two braces + single space before last brace
 
             match singleline_shape.over_budget() {
                 true => TableType::MultiLine,
