@@ -1,7 +1,8 @@
 #[cfg(feature = "luau")]
 use full_moon::ast::types::{
-    GenericDeclaration, GenericDeclarationParameter, IfExpression, IndexedTypeInfo, TypeArgument,
-    TypeAssertion, TypeField, TypeFieldKey, TypeInfo, TypeSpecifier,
+    ElseIfExpression, GenericDeclaration, GenericDeclarationParameter, GenericParameterInfo,
+    IfExpression, IndexedTypeInfo, TypeArgument, TypeAssertion, TypeField, TypeFieldKey, TypeInfo,
+    TypeSpecifier,
 };
 use full_moon::ast::{
     punctuated::Punctuated, span::ContainedSpan, BinOp, Call, Expression, FunctionArgs,
@@ -567,6 +568,9 @@ define_update_trivia!(TypeInfo, |this, leading, trailing| {
         TypeInfo::Basic(token_reference) => {
             TypeInfo::Basic(token_reference.update_trivia(leading, trailing))
         }
+        TypeInfo::String(string) => TypeInfo::String(string.update_trivia(leading, trailing)),
+        TypeInfo::Boolean(boolean) => TypeInfo::Boolean(boolean.update_trivia(leading, trailing)),
+
         TypeInfo::Callback {
             generics,
             parentheses,
@@ -603,6 +607,11 @@ define_update_trivia!(TypeInfo, |this, leading, trailing| {
             generics: generics.to_owned(),
         },
 
+        TypeInfo::GenericPack { name, ellipse } => TypeInfo::GenericPack {
+            name: name.update_leading_trivia(leading),
+            ellipse: ellipse.update_trailing_trivia(trailing),
+        },
+
         TypeInfo::Intersection {
             left,
             ampersand,
@@ -662,6 +671,11 @@ define_update_trivia!(TypeInfo, |this, leading, trailing| {
             type_info: Box::new(type_info.update_trailing_trivia(trailing)),
         },
 
+        TypeInfo::VariadicPack { ellipse, name } => TypeInfo::VariadicPack {
+            ellipse: ellipse.update_leading_trivia(leading),
+            name: name.update_trailing_trivia(trailing),
+        },
+
         other => panic!("unknown node {:?}", other),
     }
 });
@@ -743,18 +757,18 @@ define_update_leading_trivia!(GenericDeclaration, |this, leading| {
 
 #[cfg(feature = "luau")]
 define_update_leading_trivia!(GenericDeclarationParameter, |this, leading| {
-    match this {
-        GenericDeclarationParameter::Name(token) => {
-            GenericDeclarationParameter::Name(token.update_leading_trivia(leading))
-        }
-        GenericDeclarationParameter::Variadic { name, ellipse } => {
-            GenericDeclarationParameter::Variadic {
-                name: name.update_leading_trivia(leading),
-                ellipse: ellipse.to_owned(),
-            }
+    let parameter_info = match this.parameter() {
+        GenericParameterInfo::Name(token) => {
+            GenericParameterInfo::Name(token.update_leading_trivia(leading))
         }
+        GenericParameterInfo::Variadic { name, ellipse } => GenericParameterInfo::Variadic {
+            name: name.update_leading_trivia(leading),
+            ellipse: ellipse.to_owned(),
+        },
         other => panic!("unknown node {:?}", other),
-    }
+    };
+
+    this.to_owned().with_parameter(parameter_info)
 });
 
 #[cfg(feature = "luau")]
@@ -768,3 +782,9 @@ define_update_trailing_trivia!(IfExpression, |this, trailing| {
     this.to_owned()
         .with_else(this.else_expression().update_trailing_trivia(trailing))
 });
+
+#[cfg(feature = "luau")]
+define_update_leading_trivia!(ElseIfExpression, |this, leading| {
+    this.to_owned()
+        .with_else_if_token(this.else_if_token().update_leading_trivia(leading))
+});
