@@ -2,8 +2,8 @@ use crate::formatters::trivia::{FormatTriviaType, UpdateLeadingTrivia, UpdateTra
 #[cfg(feature = "luau")]
 use full_moon::ast::span::ContainedSpan;
 #[cfg(feature = "luau")]
-use full_moon::ast::types::{IndexedTypeInfo, TypeDeclaration, TypeInfo};
-use full_moon::ast::{Block, FunctionBody};
+use full_moon::ast::types::{IndexedTypeInfo, TypeArgument, TypeDeclaration, TypeInfo};
+use full_moon::ast::{Block, FunctionBody, Parameter};
 use full_moon::{
     ast::{
         BinOp, Call, Expression, Field, FunctionArgs, Index, LastStmt, Prefix, Stmt, Suffix,
@@ -430,6 +430,27 @@ pub fn take_expression_trailing_comments(expression: &Expression) -> (Expression
     )
 }
 
+pub fn take_parameter_trailing_comments(parameter: &Parameter) -> (Parameter, Vec<Token>) {
+    let trailing_trivia = match parameter {
+        Parameter::Name(token) | Parameter::Ellipse(token) => token.trailing_trivia(),
+        other => panic!("unknown node {:?}", other),
+    };
+
+    // Remove any trailing comments from the parameter if present
+    let trailing_comments: Vec<Token> = trailing_trivia
+        .filter(|token| trivia_is_comment(token))
+        .flat_map(|x| {
+            // Prepend a single space beforehand
+            vec![Token::new(TokenType::spaces(1)), x.to_owned()]
+        })
+        .collect();
+
+    (
+        parameter.update_trailing_trivia(FormatTriviaType::Replace(vec![])),
+        trailing_comments,
+    )
+}
+
 /// Macro for retrieving trailing trivia out of a stmt which ends with an `end` token
 macro_rules! end_stmt_trailing_trivia {
     ($enum:ident, $value:ident) => {{
@@ -677,6 +698,28 @@ fn get_type_info_trailing_trivia(type_info: TypeInfo) -> (TypeInfo, Vec<Token>)
     }
 }
 
+#[cfg(feature = "luau")]
+pub fn take_type_argument_trailing_comments(
+    type_argument: &TypeArgument,
+) -> (TypeArgument, Vec<Token>) {
+    let (type_info, trailing_trivia) =
+        get_type_info_trailing_trivia(type_argument.type_info().to_owned());
+
+    let trailing_comments = trailing_trivia
+        .iter()
+        .filter(|token| trivia_is_comment(token))
+        .flat_map(|x| {
+            // Prepend a single space beforehand
+            vec![Token::new(TokenType::spaces(1)), x.to_owned()]
+        })
+        .collect();
+
+    (
+        type_argument.to_owned().with_type_info(type_info),
+        trailing_comments,
+    )
+}
+
 #[cfg(feature = "luau")]
 fn get_type_declaration_trailing_trivia(
     type_declaration: TypeDeclaration,
